@@ -34,6 +34,10 @@ massage = [(re.compile('<([^<>\s]+)([^<>]*)/>'), \
 
 
 def parse_date(str):
+    """ RSS or Atom feed date parsing.
+
+    RSS and Atom feeds contains locale independent dates so it is impossible to use strptime().
+    """
     str = str.strip()
     parts = str.split(" ")
     (y, m, d, hh, mm, ss, ms) = (0, 1, 1, 0, 0, 0, 0)
@@ -79,6 +83,10 @@ def parse_date(str):
 
 
 def create_url_opener():
+    """ Opener is for opening urls.
+
+    Opener tries to use either microb (on N800) or firefox (on PC) cookies.
+    """
     filename = os.path.join(os.environ["HOME"], ".mozilla/microb/cookies.txt")
     if not os.path.exists(filename):
         files = glob.glob(os.path.join(os.environ["HOME"], ".mozilla/firefox/*default/cookies.txt"))
@@ -115,6 +123,8 @@ def get_or_create_tag(html, list, tag = None):
 
 
 def get_page_encoding(page):
+    """ Not all html documents has http-equiv meta, so this retrieves real http headers.
+    """
     if page.info().has_key("Content-Type"):
         params = map(lambda nvs: nvs.split("="), page.info()["Content-Type"].split(";"))
         charsets = filter(lambda nv: nv[0].strip() == "charset" and len(nv) > 1, params)
@@ -131,6 +141,10 @@ def get_page_base(page, html):
 
 
 def get_page_html_only(url, **kwargs):
+    """ Retrieves and parses html page.
+
+    kwargs are passed to Request object constructor.
+    """
     page = urlopener.open(urllib2.Request(url=quote_url_path(url), **kwargs))
     html = BeautifulSoup(page, fromEncoding = get_page_encoding(page))
     base = get_page_base(page, html)
@@ -141,6 +155,11 @@ refresh_reqex = re.compile("refresh", re.IGNORECASE)
 
 
 def try_skip_banner(page, html, base, **kwargs):
+    """ This may help avoid ads on some resources.
+
+    When web page contains Refresh in its meta it is likely that 
+    it is banner so we try to reload it immediately.
+    """
     refresh = None
     if page.info().has_key("Refresh"):
         refresh = page.info()["Refresh"]
@@ -172,6 +191,12 @@ def get_feed_title(xml):
 
 
 def discover_feed(url):
+    """ Returns RSS or Atom feed url from related html page.
+
+    Not only RSS or Atom feed urls may be entered by user, so
+    this function check what is passed and if it is html page
+    function try to detect feed url. It also retrieves icon.
+    """
     page = urlopener.open(url)
     icon_url = None
     content_type = None
@@ -218,6 +243,11 @@ def discover_feed(url):
 
 
 class Item:
+    """ This class groups feed element related operations together.
+
+    This is really not related to OOP best practices. It simply handy
+    to have all this fields accessible via self.
+    """
 
     page_template_default = '<html><head><meta http-equiv="Content-Type" content="text/html;charset=utf-8" ></head><body></body></html>'
     print_version_keywords_default = [u"версия для печати", u"printable version"]
@@ -227,28 +257,47 @@ class Item:
     def __init__(self, url = None, tag = None, base = None, path = "", cache_path = "", callback = None, \
         page_template = None, print_version_keywords = None, try_use_print_version = True, \
         allow_scripts = False, as_is = False, content_kwargs = None, reformatter_url = None):
-            self.__url = url
-            self.__tag = tag
-            self.__base = base
-            self.__path = path
-            self.__cache_path = cache_path
-            self.__cached = {}
-            self.__try_use_print_version = try_use_print_version
-            self.__allow_scripts = allow_scripts
-            self.__page_template = self.page_template_default
-            if page_template:
-                self.__page_template = page_template
-            self.__print_version_keywords = self.print_version_keywords_default
-            if print_version_keywords:
-                self.__print_version_keywords = print_version_keywords
-            self.__callback = callback
-            self.__attrs_for_fix = [("a", "href"), ("iframe", "src"), ("object", "data"), ("form", "action")]
-            self.__attrs_for_cache = \
-                [("applet", "codebase"), ("object", "codebase"), ("script", "src"), ("img", "src"), ("input", "src")]
-            self.title = "untitled"
-            self.__as_is = as_is
-            self.__content_kwargs = content_kwargs
-            self.__reformatter_url = reformatter_url
+        """ It is clear that this is constructor :)
+
+        Item is represented either by its content, when tag is specified, or by reference
+        (url is used in this case). 
+        base - html document base used to resolve relative references.
+        path - base directory used to store cached item
+        cache_path - subdirectory of path used to cache related resources (such as images).
+        callback - for reporting progress.
+        page_template - used for creating cached items.
+        print_version_keywords - a list of phrases to scan html document searching for reference to
+        printable version.
+        try_use_print_version - search for reference to printable version should be performed.
+        allow_scripts - do not remove script tags from item before caching.
+        as_is - do not try to reformat html document for small screens.
+        content_kwargs - it is passed to BeautifulSoup's find method. It is usually
+        tag name and its attributes.
+        reformatter_url - format string (%s is replaced with url from item).
+        Used to process html document through online services such as Skweezer.
+        """
+        self.__url = url
+        self.__tag = tag
+        self.__base = base
+        self.__path = path
+        self.__cache_path = cache_path
+        self.__cached = {}
+        self.__try_use_print_version = try_use_print_version
+        self.__allow_scripts = allow_scripts
+        self.__page_template = self.page_template_default
+        if page_template:
+            self.__page_template = page_template
+        self.__print_version_keywords = self.print_version_keywords_default
+        if print_version_keywords:
+            self.__print_version_keywords = print_version_keywords
+        self.__callback = callback
+        self.__attrs_for_fix = [("a", "href"), ("iframe", "src"), ("object", "data"), ("form", "action")]
+        self.__attrs_for_cache = \
+            [("applet", "codebase"), ("object", "codebase"), ("script", "src"), ("img", "src"), ("input", "src")]
+        self.title = "untitled"
+        self.__as_is = as_is
+        self.__content_kwargs = content_kwargs
+        self.__reformatter_url = reformatter_url
 
 
     def __report(self, action, item):
@@ -257,6 +306,10 @@ class Item:
 
 
     def __calc_the_tag(self, tag):
+        """ Searches for tag that contains useful content such as article, news etc.
+
+        Here is the primary FeedCircuit heuristic, sorta "know how".
+        """
         (textsum, tagsum, max, node) = (0, 0, 0, None)
         for text in tag.findAll(text = lambda text: not isinstance(text, Comment), recursive = False):
             textsum += len(text.strip())
@@ -304,15 +357,20 @@ class Item:
 
 
     def __extract_tag(self):
+        """ This performs main Item's job.
+
+        By extracting html document and reformatting it.
+        """
         self.__report("downloading", self.__url)
         (html, self.__base, real_url) = get_page_html(self.get_url_for_reformatter(self.__url))
-        if not self.__as_is and not self.__reformatter_url:
-            if self.__content_kwargs:
-                self.__tag = html.body.find(**self.__content_kwargs)
-            elif self.__try_use_print_version:
+        if not self.__as_is:
+            if self.__try_use_print_version:
                 (html, self.__base, self.__tag) = self.__find_print_version(html, self.__base, real_url)
-            if not self.__tag:
-                self.__tag = self.__find_the_tag(html.body)
+            if not self.__reformatter_url:
+                if self.__content_kwargs:
+                    self.__tag = html.body.find(**self.__content_kwargs)
+                if not self.__tag:
+                    self.__tag = self.__find_the_tag(html.body)
         if not self.__tag:
             self.__tag = html.body
         title = html.find("title")
@@ -353,6 +411,13 @@ class Item:
 
 
     def __fixup_tag(self):
+        """ Collection of misc postprocessing tasks.
+
+        1. Removing scripts.
+        2. Caching resources and changing references to them.
+        3. Keeping consistency by preventing extracting tags from context,
+        e.g. td should always be nested into tr etc.
+        """
         if not self.__allow_scripts:
             [script.extract() for script in self.__tag.findAll("script")]
         [self.__fix_urls(self.__tag.findAll(x[0]), x[1], self.__base) for x in self.__attrs_for_fix]
@@ -365,6 +430,7 @@ class Item:
 
 
     def save(self, title = None, filename = None):
+        """ Saves item into cache. """
         html = BeautifulSoup(self.__page_template)
         if not title:
             title = self.title
@@ -380,6 +446,7 @@ class Item:
 
 
     def process(self):
+        """ Entry point. """
         self.__contents = []
         try:
             if self.__url:
@@ -410,6 +477,7 @@ class Item:
 
 
     def delete(self, filename = None):
+        """ Deletes item and all related resources. """
         if filename:
             self.__delete_cached_item(os.path.join(self.__path, filename))
         elif self.__tag:
@@ -417,6 +485,7 @@ class Item:
 
 
 class Feed:
+    """ Collection of feed related operations. """
     
     page_item_template_default = '<b><a/></b><div></div><br>'
     item_comment_prefix = "feed item: "
@@ -426,33 +495,58 @@ class Feed:
         try_use_print_version = True, delete_before_date = None, allow_scripts = False, \
         inline_items = False, cache_items = False, include = None, exclude = None, as_is = False,
         content_kwargs = None, reformatter_url = None):
-            self.__url = url
-            self.__path = path
-            self.__cache_path = cache_path
-            self.__callback = callback
-            self.__title = title
-            self.__filename = filename
-            self.__page_template = Item.page_template_default
-            if page_template:
-                self.__page_template = page_template
-            self.__page_item_template = self.page_item_template_default
-            if page_item_template:
-                self.__page_item_template = page_item_template
-            self.__print_version_keywords = print_version_keywords
-            self.__try_use_print_version = try_use_print_version
-            self.__inline_items = inline_items
-            self.__cache_items = cache_items
-            self.__delete_before_date = delete_before_date
-            self.__allow_scripts = allow_scripts
-            self.__include = None
-            if include:
-                self.__include = re.compile(include)
-            self.__exclude = None
-            if exclude:
-                self.__exclude = re.compile(exclude)
-            self.__as_is = as_is
-            self.__content_kwargs = content_kwargs
-            self.__reformatter_url = reformatter_url
+        """ Constructor.
+
+        url - feed url.
+        path - base directory used to store cached item
+        cache_path - subdirectory of path used to cache related resources (such as images).
+        callback - for reporting progress.
+        title - if this is not provided it is tried to figure out automatically.
+        filename - if this is not provided title.html is used to store feed
+        page_template - used for creating cached items.
+        page_item_template - html used to construct feed item.
+        print_version_keywords - a list of phrases to scan html document searching for reference to
+        printable version.
+        try_use_print_version - search for reference to printable version should be performed.
+        delete_before_date - all items before this date are deleted during feed update.
+        allow_scripts - do not remove script tags from item before caching.
+        inline_items - inline referenced resources into feed html.
+        cache_items - download and cache referenced resources.
+        include - regexp for item urls to include.
+        exclude - regexp for item urls to exclude.
+        as_is - do not try to reformat html document for small screens.
+        content_kwargs - it is passed to BeautifulSoup's find method. It is usually
+        tag name and its attributes.
+        reformatter_url - format string (%s is replaced with url from item).
+        Used to process html document through online services such as Skweezer.
+        """
+        self.__url = url
+        self.__path = path
+        self.__cache_path = cache_path
+        self.__callback = callback
+        self.__title = title
+        self.__filename = filename
+        self.__page_template = Item.page_template_default
+        if page_template:
+            self.__page_template = page_template
+        self.__page_item_template = self.page_item_template_default
+        if page_item_template:
+            self.__page_item_template = page_item_template
+        self.__print_version_keywords = print_version_keywords
+        self.__try_use_print_version = try_use_print_version
+        self.__inline_items = inline_items
+        self.__cache_items = cache_items
+        self.__delete_before_date = delete_before_date
+        self.__allow_scripts = allow_scripts
+        self.__include = None
+        if include:
+            self.__include = re.compile(include)
+        self.__exclude = None
+        if exclude:
+            self.__exclude = re.compile(exclude)
+        self.__as_is = as_is
+        self.__content_kwargs = content_kwargs
+        self.__reformatter_url = reformatter_url
 
 
     def __report(self, action, item):
@@ -468,7 +562,7 @@ class Feed:
 
     def __get_item_link(self, item):
         if item.guid:
-            if not item.guid.has_key("isPermaLink") or item.guid["isPermaLink"].lower() == "true":
+            if not item.guid.has_key("ispermalink") or item.guid["ispermalink"].lower() == "true":
                 return item.guid.string
         if item.link:
             if item.link.has_key("href"):
@@ -511,6 +605,7 @@ class Feed:
 
 
     def __match_link(self, link):
+        """ Filter out link using include and exclude patterns. """
         if self.__include and not self.__include.search(link):
             return False
         if self.__exclude and self.__exclude.search(link):
@@ -519,6 +614,7 @@ class Feed:
 
 
     def __create_feed_item(self, url = None, tag = None, base = None, path = None, cache_path = None):
+        """ This is just not to type all this code when you need new Item instance. """
         if path == None:
             path = self.__path
         if cache_path == None:
@@ -530,6 +626,7 @@ class Feed:
         
 
     def __add_item(self, div, item):
+        """ Here Item is get included to feed after executing all dependant operations. """
         link = self.__get_item_link(item)
         if self.__match_link(link) and not self.__outdated(item) and not self.__item_exists(div.parent, link):
             title = self.__unquote_special(item.title.string.strip())
@@ -575,6 +672,11 @@ class Feed:
 
 
     def update(self):
+        """ Updates feed and returns numbers of new items.
+
+        When there are now new items feed file stays untouched.
+        Outdated items are deleted only when there are some new ones.
+        """
         result = 0
         try:
             self.__report("downloading", self.__url)
@@ -614,6 +716,7 @@ class Feed:
 
 
     def delete(self):
+        """ Deletes everything related to feed from cache. """
         if self.__filename:
             path = os.path.join(self.__path, self.__filename)
         else:
