@@ -17,13 +17,14 @@ import feedcircuit
 import urllib
 import webbrowser
 import glob
+import shutil
 
 gtk.gdk.threads_init()
 
 #This is to be able to start Feed Circuit on PC
 try:
     import osso
-    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.5", False)
+    osso_context = osso.Context("org.maemo.feedcircuit", "0.6.6", False)
 except ImportError:
     pass
 
@@ -371,6 +372,26 @@ class FeedListModel(gtk.GenericTreeModel):
             feed_config.cache_path = cache_path
 
 
+    def move(self, id, pos):
+        before_id = None
+        if pos >= 0 and pos < len(self.list):
+            before_id = self.list[pos].id
+        if before_id == id:
+            return
+        old_order = [f.id for f in self.list]
+        index = self.map[id]
+        feed_config = self.list[index]
+        del self.list[index]
+        self.rebuild_map()
+        if before_id:
+            self.list.insert(self.map[before_id], feed_config)
+        else:
+            self.list.append(feed_config)
+        self.rebuild_map()
+        self.rows_reordered(None, None, [self.map[id] for id in old_order])
+        self.save_config()
+
+
 class PreferencesWindow(gtk.Dialog):
 
 
@@ -664,6 +685,15 @@ class Program(hildon.Program):
 
         self.__view.connect("row_activated", self.row_activated)
 
+        dnd_targets = [("FeedCircuitFeed", gtk.TARGET_SAME_WIDGET, 0)]
+        self.__view.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, dnd_targets, gtk.gdk.ACTION_MOVE)
+        self.__view.drag_source_set(gtk.gdk.BUTTON1_MASK, dnd_targets, gtk.gdk.ACTION_MOVE)
+        self.__view.enable_model_drag_dest(dnd_targets, gtk.gdk.ACTION_MOVE)
+        self.__view.connect("drag-data-get", self.drag_data_get)
+        self.__view.connect("drag-data-received", self.drag_data_received)
+        self.__view.connect("drag-drop", self.drag_drop)
+
+
         sw = gtk.ScrolledWindow()
         sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         sw.add(self.__view)
@@ -699,6 +729,30 @@ class Program(hildon.Program):
             self.__window_in_fullscreen = False
 
 
+    def drag_drop(self, widget, drag_context, x, y, timestamp):
+        self.__view.emit_stop_by_name('drag-drop')
+        self.__view.drag_get_data(drag_context, drag_context.targets[0], timestamp)
+        return 1
+
+
+    def drag_data_get(self, widget, drag_context, selection_data, info, timestamp):
+        feed_configs = self.get_selected_feeds()
+        if len(feed_configs):
+            id = feed_configs[0].id
+            selection_data.set(selection_data.target, len(id), id)
+
+
+    def drag_data_received(self, widget, drag_context, x, y, selection_data, info, timestamp):
+        self.__view.emit_stop_by_name('drag-data-received')
+        pos = None
+        row_data = self.__view.get_dest_row_at_pos(x, y)
+        if row_data:
+            pos = row_data[0][0]
+            if row_data[1] == gtk.TREE_VIEW_DROP_AFTER:
+                pos += 1
+        self.__model.move(selection_data.data, pos)
+
+
     def run(self):
         self.__window.show_all()
         gtk.main()
@@ -894,16 +948,13 @@ class Program(hildon.Program):
 
 
     def move_cache(self, from_path, to_path):
-        to_path_cache = os.path.join(to_path, ".cache")
-        if not os.path.exists(to_path_cache):
-            os.makedirs(to_path_cache)
+        if not os.path.exists(to_path):
+            os.makedirs(to_path)
         for file in glob.glob(os.path.join(from_path, "*")):
             filename = os.path.split(file)[1]
             if filename != "config":
-                os.rename(file, os.path.join(to_path, filename))
-        for file in glob.glob(os.path.join(from_path, ".cache/*")):
-            os.rename(file, os.path.join(to_path_cache, os.path.split(file)[1]))
-        os.rmdir(os.path.join(from_path, ".cache"))
+                shutil.move(file, os.path.join(to_path, filename))
+        shutil.move(os.path.join(from_path, ".cache"), os.path.join(to_path, ".cache"))
 
 
     def preferences(self, source):
@@ -929,7 +980,7 @@ class Program(hildon.Program):
     def about(self, source):
         dialog = gtk.AboutDialog()
         dialog.set_name("FeedCircuit")
-        dialog.set_version("0.6.5")
+        dialog.set_version("0.6.6")
         dialog.set_website("http://feedcircuit.garage.maemo.org")
         dialog.set_authors([_("George Kibardin")])
         dialog.run()
